package org.japura.task;

import java.util.ArrayList;
import java.util.List;

import org.japura.MethodNotImplementedException;
import org.japura.controller.Group;
import org.japura.debug.DebugComponent;
import org.japura.debug.DebugResult;

/**
 * <P>
 * Copyright (C) 2012-2013 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
public class MockTaskManager extends AbstractTaskManager{

  private List<Task<?>> executedTasks = new ArrayList<Task<?>>();

  private void execute(Task<?> currentTask) {
	executedTasks.add(currentTask);

	fireTaskManagerListeners(TaskManagerEvent.SUBMITTED, currentTask);
	fireTaskExecutionUIs(TaskManagerEvent.SUBMITTED, currentTask);
	fireTaskManagerListeners(TaskManagerEvent.BEFORE_EXECUTE, currentTask);
	fireTaskExecutionUIs(TaskManagerEvent.BEFORE_EXECUTE, currentTask);

	if (currentTask.isCanceled()) {
	  currentTask.canceled();
	  fireTaskManagerListeners(TaskManagerEvent.AFTER_EXECUTE, currentTask);
	  fireTaskExecutionUIs(TaskManagerEvent.AFTER_EXECUTE, currentTask);
	} else {
	  try {
		currentTask.doInBackground();
	  } catch (Exception e) {
		currentTask.handleException(e);
	  }
	  currentTask.done();
	  fireTaskManagerListeners(TaskManagerEvent.AFTER_EXECUTE, currentTask);
	  fireTaskExecutionUIs(TaskManagerEvent.AFTER_EXECUTE, currentTask);

	  for (Task<?> nestedTask : currentTask.getNestedTasks()) {
		nestedTask.setRootModel(currentTask.getRootModel());
		nestedTask.setSession(currentTask.getSession());
		if (nestedTask.getMessage() == null) {
		  nestedTask.setMessage(currentTask.getMessage());
		}
		if (currentTask.hasException()) {
		  nestedTask.cancel();
		} else {
		  nestedTask.setCanceled(false);
		}
		execute(nestedTask);
	  }
	}
  }

  @Override
  public void submit(Task<?> task, TaskSession session, boolean isParallel) {
	session = getTaskSessionFactory().buildTaskSession(task, session);
	checkSession(task, session);
	task.setSession(session);
	executedTasks = new ArrayList<Task<?>>();
	execute(task);
  }

  @Override
  public boolean hasTask(Group group) {
	return false;
  }

  @Override
  public boolean hasTask() {
	return false;
  }

  @Override
  public void cancel(String controllerGroupId) {
	// TODO ops
  }

  @Override
  public void cancelAll() {
	// TODO ops
  }

  @Override
  public int getQueueCount() {
	return 0;
  }

  @Override
  public DebugComponent createDebugPanel() {
	throw new MethodNotImplementedException();
  }

  @Override
  public void showDebugWindow() {
	throw new MethodNotImplementedException();
  }

  public List<Task<?>> getExecutedTasks() {
	return executedTasks;
  }

  @Override
  public List<DebugResult> addToDebugWindow(Task<?> task, TaskEvent event) {
	return new ArrayList<DebugResult>();
  }

  @Override
  public void addToDebugWindow(Task<?> task) {}

  @Override
  public void removeFromDebugWindow(Task<?> task) {}

  @Override
  public void notifyAfterTaskExecution(Task<?> task) {}

}
