package org.japura.task.ui;

import java.util.HashMap;

import org.japura.controller.Controller;
import org.japura.task.Task;

/**
 * <P>
 * Copyright (C) 2012-2013 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
public class DefaultTaskExecutionUI implements TaskExecutionUI{

  private GroupUIFactory groupUIFactory = new DefaultGroupUIFactory();

  private HashMap<String, Integer> tasksCount;
  private int globalTasksCount;

  private TaskExecutionUI globalTaskExecutionUI = null;
  private HashMap<String, TaskExecutionUI> groupUIs;

  public DefaultTaskExecutionUI() {
	groupUIs = new HashMap<String, TaskExecutionUI>();
	tasksCount = new HashMap<String, Integer>();
  }

  @Override
  public synchronized void submitted(Task<?> task) {
	if (task.isCanceled()) {
	  return;
	}

	TaskExecutionUI taskExecutionUI = null;
	Controller<?> controller = getController(task);
	if (controller != null) {
	  String groupId = controller.getGroupId();

	  Integer count = tasksCount.get(groupId);
	  if (count == null) {
		count = 0;
	  }
	  count++;
	  tasksCount.put(groupId, count);

	  taskExecutionUI = groupUIs.get(groupId);
	  if (taskExecutionUI == null) {
		taskExecutionUI = getGroupUIFactory().buildTaskExecutionUI(groupId);
		if (taskExecutionUI != null) {
		  groupUIs.put(groupId, taskExecutionUI);
		}
	  }
	} else {
	  globalTasksCount++;
	  taskExecutionUI = globalTaskExecutionUI;
	}

	if (taskExecutionUI != null) {
	  taskExecutionUI.submitted(task);
	}
  }

  @Override
  public synchronized void beforeExecute(Task<?> task) {
	TaskExecutionUI taskExecutionUI = null;

	Controller<?> controller = getController(task);
	if (controller != null) {
	  String groupId = controller.getGroupId();
	  taskExecutionUI = groupUIs.get(groupId);
	} else {
	  taskExecutionUI = globalTaskExecutionUI;
	}

	if (taskExecutionUI != null) {
	  taskExecutionUI.beforeExecute(task);
	}
  }

  @Override
  public synchronized void afterExecute(Task<?> task) {
	TaskExecutionUI taskExecutionUI = null;

	Controller<?> controller = getController(task);
	if (controller != null) {
	  String groupId = controller.getGroupId();
	  taskExecutionUI = groupUIs.get(groupId);

	  Integer count = tasksCount.get(groupId);
	  count--;
	  if (count == 0) {
		tasksCount.remove(groupId);
		groupUIs.remove(groupId);
	  } else {
		tasksCount.put(groupId, count);
	  }
	} else {
	  taskExecutionUI = globalTaskExecutionUI;
	  globalTasksCount--;
	  if (globalTasksCount == 0) {
		globalTaskExecutionUI = null;
	  }
	}

	if (taskExecutionUI != null) {
	  taskExecutionUI.afterExecute(task);
	}
  }

  private Controller<?> getController(Task<?> task) {
	if (task.getRootModel() != null) {
	  Controller<?> c = task.rootModelTo(Controller.class);
	  if (c != null && Controller.contains(c)) {
		return c;
	  }
	}
	return null;
  }

  public TaskExecutionUI getGlobalTaskExecutionUI() {
	return globalTaskExecutionUI;
  }

  public void setGlobalTaskExecutionUI(TaskExecutionUI taskExecutionUI) {
	this.globalTaskExecutionUI = taskExecutionUI;
  }

  public void setGroupUIFactory(GroupUIFactory factory) {
	this.groupUIFactory = factory;
  }

  public GroupUIFactory getGroupUIFactory() {
	return groupUIFactory;
  }

  public static class DefaultGroupUIFactory implements GroupUIFactory{
	@Override
	public TaskExecutionUI buildTaskExecutionUI(String groupId) {
	  return new ModalProgressBarUI(groupId);
	}
  }
}
