package org.japura.task.ui;

import javax.swing.Icon;
import javax.swing.JLayeredPane;

import org.japura.controller.Controller;
import org.japura.controller.ControllerModel;
import org.japura.gui.ExecutionPanel;
import org.japura.task.Task;

/**
 * <P>
 * Copyright (C) 2012-2013 Carlos Eduardo Leite de Andrade
 * <P>
 * This library is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * <P>
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * <P>
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <A
 * HREF="www.gnu.org/licenses/">www.gnu.org/licenses/</A>
 * <P>
 * For more information, contact: <A HREF="www.japura.org">www.japura.org</A>
 * <P>
 * 
 * @author Carlos Eduardo Leite de Andrade
 */
public class ModalProgressBarUI extends AbstractTaskExecutionUI{

  private String groupId;
  private Icon progressBarIcon;
  private ExecutionPanel panel;

  public ModalProgressBarUI(String groupId) {
	this(groupId, null);
  }

  public ModalProgressBarUI(String groupId, Icon progressBarIcon) {
	this.groupId = groupId;
	this.progressBarIcon = progressBarIcon;
  }

  @Override
  protected void initialize() {
	panel = new ExecutionPanel("", progressBarIcon);
  }

  @Override
  protected void delayedStart() {
	ControllerModel controller = Controller.getRoot(groupId);
	if (controller != null) {
	  controller.addModal(panel, null, JLayeredPane.POPUP_LAYER + 1);
	  panel.start();
	}
  }

  @Override
  public void update(Task<?> task) {
	if (task.getMessage() != null) {
	  panel.setText(task.getMessage());
	}
  }

  @Override
  protected void finish() {
	panel.stop();
	ControllerModel rootController = Controller.getRoot(groupId);
	if (rootController != null) {
	  rootController.closeModal(panel);
	}
	panel = null;
  }
}
